<?php

namespace App\Tests\Domain\Registry\Symfony\EventSubscriber\Doctrine;

use App\Domain\Registry\Calculator\ConformiteOrganisationConformiteCalculator;
use App\Domain\Registry\Model\ConformiteOrganisation\Evaluation;
use App\Domain\Registry\Symfony\EventSubscriber\Doctrine\ConformiteOrganisationSubscriber;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Event\PrePersistEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;
use PHPUnit\Framework\TestCase;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;

class ConformiteOrganisationSubscriberTest extends TestCase
{
    use ProphecyTrait;

    /**
     * @var ConformiteOrganisationConformiteCalculator|ObjectProphecy
     */
    private $calculator;

    /**
     * @var ConformiteOrganisationSubscriber
     */
    private $subscriber;

    private $entityManager;

    public function setUp(): void
    {
        $this->calculator = $this->prophesize(ConformiteOrganisationConformiteCalculator::class);

        $this->subscriber    = new ConformiteOrganisationSubscriber($this->calculator->reveal());
        $this->entityManager = $this->prophesize(EntityManagerInterface::class);
    }

    public function testInstanceOf()
    {
        $this->assertInstanceOf(EventSubscriber::class, $this->subscriber);
    }

    public function testGetSubscribedEvents()
    {
        $this->assertEquals(
            [
                'prePersist',
                'preUpdate',
            ],
            $this->subscriber->getSubscribedEvents()
        );
    }

    public function testPrePersist()
    {
        $object = new Evaluation();

        $eventArgs = new PrePersistEventArgs($object, $this->entityManager->reveal());

        $this->calculator->calculEvaluationConformites($object)->shouldBeCalled();

        $this->subscriber->prePersist($eventArgs);
    }

    public function testPreUpdate()
    {
        $object    = new Evaluation();
        $a         = [];
        $eventArgs = new PreUpdateEventArgs($object, $this->entityManager->reveal(), $a);

        $this->calculator->calculEvaluationConformites($object)->shouldBeCalled();

        $this->subscriber->preUpdate($eventArgs);
    }
}
